from ftw.solr.testing import SOLR_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest2 import TestCase


class TestReindexObjectSecurityPatches(TestCase):

    layer = SOLR_FUNCTIONAL_TESTING

    def test_CatalogAware_reindexObjectSecurity_is_patched(self):
        from Products.CMFCore.CMFCatalogAware import CatalogAware
        self.assertEquals(CatalogAware.reindexObjectSecurity.__func__.func_name,
                          'ftw_solr_CatalogAware_reindexObjectSecurity')



from collective.solr.manager import SolrConnectionManager
from collective.solr.search import Search
from collective.solr.tests.utils import getData, fakehttp
from ftw.solr.patches.search import buildQueryAndParameters
from ftw.solr.testing import SOLR_INTEGRATION_TESTING
from unittest import TestCase


class TestBuildQuery(TestCase):

    layer = SOLR_INTEGRATION_TESTING

    def setUp(self):

        self.mngr = SolrConnectionManager()
        self.mngr.setHost(active=True)
        conn = self.mngr.getConnection()
        fakehttp(conn, getData('plone_schema.xml'))  # fake schema response
        self.mngr.getSchema()                       # read and cache the schema
        self.search = Search()
        self.search.manager = self.mngr

        # Patch buildQuery method
        self.search.buildQuery = buildQueryAndParameters.__get__(
            self.search,
            self.search.__class__)

    def tearDown(self):
        self.mngr.closeConnection()
        self.mngr.setHost(active=False)

    def build_query(self, *args, **kwargs):
        query = self.search.buildQuery(*args, **kwargs)[0]
        return ' '.join(sorted(query.values()))

    def test_simple_path_query(self):
        self.assertEquals('+path_parents:\\/spam\\/and\\/eggs',
                          self.build_query(path_parents='/spam/and/eggs'))

    def test_path_list_query(self):
        self.assertEquals('+path_parents:("\\/spam" OR "\\/eggs")',
                          self.build_query(path_parents=['/spam', '/eggs']))

    def test_slash_in_searchabletext(self):
        self.assertEquals('+SearchableText:spam\\/eggs',
                          self.build_query(SearchableText='spam/eggs'))

    def test_queries_with_path_and_depth_should_be_escaped(self):
        self.assertEquals(
            '+path_depth:[4 TO 6] AND +path_parents:\\/spam\\/and\\/eggs',
            self.build_query(
                path_parents=set(['+path_depth:[4 TO 6] AND '
                                  '+path_parents:/spam/and/eggs'])))



from ftw.solr.behaviors import ISearchwords
from ftw.upgrade import UpgradeStep
from plone.dexterity.interfaces import IDexterityContent


class UpdateSearchwordsIndex(UpgradeStep):
    """Update searchwords index.
    """

    def __call__(self):
        query = {'object_provides': {
            'query': [ISearchwords.__identifier__,
                      IDexterityContent.__identifier__],
            'operator': 'and'}}
        for obj in self.objects(query, 'Update searchwords index'):
            if ISearchwords(obj).searchwords:
                obj.reindexObject(idxs=['searchwords'])
        self.install_upgrade_profile()



from plone import api
from plone.indexer import indexer
from plone.indexer.interfaces import IIndexer
from plone.indexer.wrapper import IndexableObjectWrapper
from Products.CMFCore.interfaces import IContentish
from Products.CMFCore.utils import getToolByName
from Products.PluginIndexes.common import safe_callable
from Products.ZCatalog.interfaces import IZCatalog
from zope.component import adapts
from zope.component import getMultiAdapter
from zope.interface import implements
from zope.interface import Interface
import re


@indexer(Interface)
def site_area(obj):
    """ Index the title of the closest navigation root
    """
    return api.portal.get_navigation_root(obj).Title()


@indexer(Interface)
def site_section(obj, **kwargs):
    """An indexer which returns the section title for a content item.
    """
    portal_state = getMultiAdapter((obj, obj.REQUEST),
                                   name=u'plone_portal_state')
    navroot = portal_state.navigation_root()
    contentPath = obj.getPhysicalPath()[len(navroot.getPhysicalPath()):]
    if contentPath:
        section = navroot.unrestrictedTraverse(contentPath[0])
        return section.Title()
    return ''


@indexer(Interface)
def creator_fullname(obj, **kwargs):
    """An indexer which returns the full name of the content creator.
    """
    creator = obj.Creator()
    mtool = getToolByName(obj, 'portal_membership')
    member_info = mtool.getMemberInfo(creator)
    if member_info and member_info['fullname']:
        return member_info['fullname']
    return creator


class SnippetTextIndexer(object):
    """Text for snippets (aka highlighting) in search results.
       Uses the SearchableText but excludes some fields that should not be
       shown in search results.
    """
    implements(IIndexer)
    adapts(Interface, IZCatalog)

    def __init__(self, context, catalog):
        self.context = context
        self.catalog = catalog

    def __call__(self):
        from collective.dexteritytextindexer import indexer
        text = indexer.dynamic_searchable_text_indexer(self.context)()

        # Remove title
        text = text.replace(self.context.Title(), '', 1)

        return text
